#!/usr/bin/python
#
# $Header$
#
# Python rolodex partial implementation.
#
# Author: Erik Nygren <nygren@mit.edu>
#


import re, os, sys, string, stat, anydbm, whichdb


class FileWatch:
    """ Used to tell if a file has been modified. """
    def __init__(self, filename):
        self.filename = filename
        self.update()

    def update(self):
        self.last_mtime = self.last_modified()

    def last_modified(self):
        statresults = os.stat(self.filename)
        return statresults[stat.ST_MTIME]
    
    def has_changed(self):
        """ Has this file changed since the last invokation?
            Updates the last modified timestamp... """
        lm = self.last_modified()
        if (lm != self.last_mtime):
            self.last_mtime = lm
            return 1
        else:
            return 0

class Rolodex:
    """ Base class for rolodex operations. """

    def check_reload(self): return 0
    def load_database(self): pass


class RolodexDB(Rolodex):
    """
    Loads a database file of prefixes.
    Doesn't yet support filewatch.
    """

    def __init__(self, filename):
        self.filename = filename
        self.db = anydbm.open(filename, "r")

    def find_number_matches(self, number):
        try:
            (a,b,c) = string.split(number, "-")
        except ValueError: return []
        m = []
        if self.db.has_key(a): m.append(self.db[a])
        if self.db.has_key(a+"-"+b): m.append(self.db[a+"-"+b])
        if self.db.has_key(a+"-"+b+"-"+c): m.append(self.db[a+"-"+b+"-"+c])
        return m


class RolodexFile(Rolodex):
    """
    Loads in a single rolodex file for operations.
    self.db is a list of (regex, value) pairs 
    self.timestamp is the timestamp of the db file
    self.filename is the name of the db file
    self.fw watches the file for modifications
    """

    def __init__(self, filename):
        self.fw = FileWatch(filename)
        self.filename = filename
        self.load_database()
    
    def load_database(self):
        self.db = []
        self.fw.update()
        f = open(self.filename, "r")
        x = f.readline()
        while (x):
            if (x[0] == "#" or x[0:3] == "---"
                 or x[0] == "\n" or x[0] == " "):
                pass
            else:
                (pattern, where) = string.split(x, " ", 1)
                self.db.append((re.compile(pattern), string.strip(where)))
            x = f.readline()

    def find_number_matches(self, number):
        "Returns a list of all matches for number"
        m = []
        for entry in self.db:
            if entry[0].match(number):
                m.append(entry[1])
        return m

    def check_reload(self):
        "Checks if a reload is necessary.  Returns 1 if db is reloaded."
        if self.fw.has_changed():
            self.load_database()
            return 1
        else:
            return 0


class RolodexMultiFileCache(Rolodex):

    """ Loads multiple rolodex database files
    and also caches search results. """    

    def __init__(self, filenames):
        self.cache = {}
        self.dbs = []
        for f in filenames:
            if whichdb.whichdb(f):
                self.dbs.append(RolodexDB(f))
            else:    
                self.dbs.append(RolodexFile(f))

    def check_reload(self):
        "Attempt to reload databases and clear the cache if any change"
        anychanges = 0
        for db in self.dbs:
            if db.check_reload():
                anychanges = 1
        if anychanges:
            self.cache = {}

    def find_number_matches(self, number):
        "Returns a list of all matches for number"
        if (self.cache.has_key(number)):
            return self.cache[number][:]
        m = []
        for db in self.dbs:
            results = db.find_number_matches(number)
            if (results):
                m.extend(results)
        self.cache[number] = m
        return m


if (__name__ == "__main__"):
    print "about to load dbs %s"%(sys.argv[1:-1])
    r = RolodexMultiFileCache(sys.argv[1:-1])
    print "loaded db"
    print r.find_number_matches(sys.argv[-1])

